function scaled = blockScale(img,n)
    %image is square and even sized, scaling factor n is odd. n poistive: 
    %upscales it nearest neighbor to n*size(img); n negative: 1/n*size(img)
    %and the image must have first been upsized by the same n
    N = length(img);
    
    if mod(size(img),2)
        disp('Terminating: Input image size must be NxN even.')
        return
    end
    if ~mod(n,2)
        disp('Terminating: Rescaling factor n must be positive or negative odd')
        return
    end
    
    if n < 0                            %downscale
        if mod(N/n,1)
            disp('Terminating: image not previously upscaled by n')
            return
        end
        n = -n;
        scaled = zeros(N/n);
        i = 1; j = 1;
        for c = 1:n:N
            for r = 1:n:N
                scaled(i,j) = mean2(img(r:r+n-1,c:c+n-1));
                i = i + 1;
            end
            i = 1;
            j = j + 1;
        end
    else                                %upscale
        scaled = zeros(N*n);
        i = ceil(n/2); j = ceil(n/2);
        fl = floor(n/2);
        for c = 1:N
            for r = 1:N
                scaled(i-fl:i+fl,j-fl:j+fl) = img(r,c);
                i = i + n;
            end
            i = ceil(n/2);
            j = j + n;
        end
    end
end