%Extracts OTF from blurred and noisy simulated Dead Leaves image (img)
%starting with ideal target image (o). Compares radial mean reasults from
%three methods.  MTF=|OTF|, PSF=ifft2(OTF).
%https://github.com/gpeyre/matlab-toolboxes/blob/master/toolbox_image/compute_dead_leaves_image.m
scaleFactor = 5;                            %for lens blur 'continuous'
N = 512;                                    %square image size
targetn = compute_dead_leaves_image(N,3);   %ideal target, linear 0->1
k = 3;                                      %power law of above image

targetn = 0.25 + targetn * .5;              %lower contrast
                                            %noise model:
FWC = 8192;                                 %e-
RN = 3;                                     %e- rms
PRNU = 0.005;                               %0.5%
GAIN = 0.47;                                %DN/e-
FPN = randn(N) * PRNU;                      %fixed non uniformities

n = 0;                                      %stops below full scale
fs = FWC /(2^n);                            %e-, full scale
o = targetn * fs;                           %e-, ideal reference image

blurRadius = 1;                             %px, lens blur
img = blockScale(o,scaleFactor);            %upscale to avoid aliasing
img = imgaussfilt(img,blurRadius*scaleFactor,'Padding','Symmetric');
img = blockScale(img,-scaleFactor);         %downscale back
                                            %add noise
img = poissrnd(img);                        %e-, ideal img + shot noise
img = img .* (1 + FPN);                     %e-, + PRNU
img = img + randn(N) * RN;                  %e-, + Read Noise

I = fftshift(fft2(img)) / mean2(img) / N^2; %FT of image corrupted by blur+noise
O = fftshift(fft2(o)) / mean2(o) / N^2;     %FT of uncorrupted image
%% Transfer Function Direct

OTF1 = I ./ O;

%% MTF per CAO et al. via Power Spectrum
%https://corp.dxomark.com/wp-content/uploads/2017/11/Dead_Leaves_Model_EI2010.pdf
[X,Y] = meshgrid(-N/2:N/2-1);
T = 1 ./ sqrt(X.^2 + Y.^2);                 %inverse distance from center
% T(N^2/2+N/2+1) = 1;                       %correct divide by zero

MTF2 = sqrt( I.*conj(I) ./ T.^k );

%% Transfer Function per Kirk et al. via Cross Power Density
%https://www.image-engineering.de/content/library/conference_papers/2014_03_15/EIC2014_9023-11.pdf

OTF3 = I .* conj(O) ./ T.^k ;

%% Plot
[y1,r] = radialMean(OTF1);                  %Compute radial average
[y2,~] = radialMean(MTF2);
[y3,~] = radialMean(OTF3);
                                            %scaling coefficients
c2 = mean(abs(y1(25:75))) / mean(abs(y2(25:75))); %25:35 = good linearity
c3 = mean(abs(y1(25:75))) / mean(abs(y3(25:75))); %of target with 1/T^k

cpp = single(r)/N;                          %cycles per pixel

plot( cpp,real(y1), cpp,real(y2)*c2, cpp,real(y3)*c3, ...
      cpp,exp(-1/2*(2*pi*cpp).^2 * blurRadius^2))   %gaussian reference

ylim([0 1.1]); grid on; 
legend('Direct FT Ratio','$\sqrt{SPD}$ Ratio','CPD Ratio',...
    'Gaussian Ref','Interpreter','Latex')
xlabel('c/p'); ylabel('MTF, Radial Mean')
title(['Texture MTF Comparison - ' num2str(n) ' stops below full scale'])
    
